

#import matplotlib
#import matplotlib.pyplot as plt
#from matplotlib.backends.backend_agg import FigureCanvasAgg


#import os
import ctypes
import numpy as np

from multiprocessing import Process, Pipe, Array, Value

def tonumpyarray(shared_mem_array):
    return np.frombuffer(shared_mem_array.get_obj(), dtype=np.uint8)

def plot_receiver(connection, shared_mem_for_plot, shared_mem_trainId):
    '''
    this function makes the plot for the single shot contrast'''
    print('Receiver: Running', flush=True)
    # consume work

    
    while True:
        # get a unit of work
        item = connection.recv()

        if item is None:
            break
        
        print("PLOT RECEIVER RECEIVED DATA, CALLONG FUN")
        output = item['fun'](item['arg']) 
        print("PLOT RECEIVER RECEIVED DATA, FUN EXECUTED, YAY!")
        npArray = tonumpyarray(shared_mem_for_plot)
        npArray[:] = output.reshape(-1)
        shared_mem_trainId.value = item['train_id']
        
    # all done
    print('Receiver: Done', flush=True)


class OnlineImagePlotter():

    def __init__(self, figsize = (15,10)):
        self.figsize = figsize
        arraysize_bytes = (self.figsize[0]*100)*(self.figsize[1]*100)*4

        self.plot_memory = Array(ctypes.c_ubyte, [0]*arraysize_bytes)
        self.train_id =  Value(ctypes.c_ulonglong, 0)
        self.prev_train_id = -1

        #pipe
        receiver, sender = Pipe()
        self.receiver = receiver
        self.sender = sender

        self.process = Process(target=plot_receiver, args=(self.receiver, self.plot_memory, self.train_id), daemon=True)
        self.process.start()

    def get_image_buffer_if_ready_and_make_new_image(self, train_id, fun, arg):
        '''
        return None if image is not ready
        return np array of shape (fihsize[1]*100, figsize[1]*100, 4) otherwise
        which then can be used in the output hash
        -> output_hash[' hash name '] = ImageData(np array from this function)
        '''

        current_image = None

        def is_new_image_ready():
            return self.prev_train_id != self.train_id.value

        if is_new_image_ready():
            
            print("NEW IMAGE READY", "from tid", self.prev_train_id, "asking new for tid", train_id)
            # get the previously computed image

            def get_plot_as_np_array(shared_mem_array):
                np_array = tonumpyarray(shared_mem_array)
                return np.reshape(np_array, (self.figsize[1]*100, self.figsize[0]*100, 4))
            
            current_image = get_plot_as_np_array(self.plot_memory)

            self.prev_train_id = self.train_id.value

            # ask for next image to be made
            package = dict()
            package['train_id'] = train_id
            package['fun'] = fun
            package['arg'] = arg
            self.sender.send(package)

        return current_image
